import { FileStreamOpt, StdoutStreamOpt } from '../streams';
export interface Mapper {
    stdio: StdoutStreamOpt;
    file: FileStreamOpt;
}
export interface BaseProcessOpts<T extends keyof Mapper> {
    since?: string;
    cwd?: string;
    output?: boolean;
    stream: {
        name?: T;
        opts?: T extends 'stdio' ? StdoutStreamOpt : FileStreamOpt;
    };
}
export interface FormatData {
    total: {
        increLine: number;
        covLine: number;
        increRate: string;
    };
    files: {
        name: string;
        increLine?: number;
        covLine?: number;
        increRate?: string;
        detail?: {
            number: number;
            hits: number;
        }[];
    }[];
}
export declare class BaseProcess<T extends keyof Mapper> {
    lcovPath: string;
    opts: BaseProcessOpts<T>;
    private lcov;
    private firstGitMessage;
    private diffData;
    private formatData;
    constructor(lcovPath: string, opts?: BaseProcessOpts<T>);
    exec(): Promise<FormatData>;
    private getLcov;
    private getLog;
    private getDiff;
    private format;
    private output;
}
export * from './async';
export * from './sync';
