"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !exports.hasOwnProperty(p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseProcess = void 0;
const os_1 = __importDefault(require("os"));
const dayjs_1 = __importDefault(require("dayjs"));
const parsers_1 = require("../parsers");
const streams_1 = require("../streams");
class BaseProcess {
    constructor(lcovPath, opts = { stream: {} }) {
        this.opts = {
            stream: {},
        };
        this.lcov = {};
        this.firstGitMessage = {};
        this.diffData = [];
        this.formatData = {
            total: { increLine: 0, covLine: 0, increRate: '' },
            files: [],
        };
        if (typeof lcovPath !== 'string') {
            throw new Error('请传递 lcov 文件路径');
        }
        this.lcovPath = lcovPath;
        const startDay = dayjs_1.default().startOf('month').format('YYYY-MM-DD');
        this.opts.cwd = opts.cwd || process.cwd();
        this.opts.since = opts.since || startDay;
        this.opts.stream.name = opts.stream.name || 'file';
        this.opts.stream.opts = opts.stream.opts;
    }
    exec() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getLcov();
            yield this.getLog();
            yield this.getDiff();
            this.format();
            if (this.opts.output) {
                this.output();
            }
            return this.formatData;
        });
    }
    getLcov() {
        return __awaiter(this, void 0, void 0, function* () {
            this.lcov = yield new parsers_1.LcovParser(this.lcovPath).run();
        });
    }
    getLog() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield new parsers_1.LogParser({
                repo: this.opts.cwd,
                since: this.opts.since,
            }).run();
            this.firstGitMessage = res[res.length - 1];
        });
    }
    getDiff() {
        return __awaiter(this, void 0, void 0, function* () {
            this.diffData = yield new parsers_1.DiffParser(this.firstGitMessage.hash, {
                cwd: this.opts.cwd,
            }).run();
            if (os_1.default.platform() === 'win32') {
                this.diffData = this.diffData.map((item) => {
                    return Object.assign(Object.assign({}, item), { newPath: item.newPath.replace(/\\/g, '/') });
                });
            }
        });
    }
    format() {
        Object.keys(this.lcov).forEach((lcovItem) => {
            this.diffData.forEach((diffItem) => {
                if (lcovItem.toLocaleLowerCase().includes(diffItem.newPath.toLocaleLowerCase())) {
                    const info = this.lcov[lcovItem];
                    const temp = { increLine: 0, covLine: 0, increRate: '', detail: [], name: lcovItem };
                    const diffLineArr = [];
                    diffItem.hunks.forEach((hunk) => {
                        hunk.changes.forEach((change) => {
                            if (change.type === 'insert' && change.lineNumber) {
                                diffLineArr.push(change.lineNumber);
                            }
                        });
                    });
                    const lcovLineHit = {};
                    const lcovLineArr = [];
                    const details = info.lines;
                    details.forEach((detail) => {
                        const { hits } = detail;
                        const { number } = detail;
                        lcovLineHit[number] = hits;
                        lcovLineArr.push(parseInt(number, 10));
                    });
                    const diffLineArrSet = new Set(diffLineArr);
                    const intersectArr = lcovLineArr.filter((x) => diffLineArrSet.has(x));
                    intersectArr.forEach((line) => {
                        this.formatData.total.covLine += lcovLineHit[line] > 0 ? 1 : 0;
                        temp.covLine += lcovLineHit[line] > 0 ? 1 : 0;
                        temp.detail.push({
                            number: line,
                            hits: lcovLineHit[line],
                        });
                    });
                    this.formatData.total.increLine += intersectArr.length;
                    temp.increLine += intersectArr.length;
                    temp.increRate = `${((temp.covLine / temp.increLine) * 100).toFixed(2)}%`;
                    this.formatData.files.push(temp);
                }
            });
        });
        this.formatData.total.increRate = `${((this.formatData.total.covLine / this.formatData.total.increLine) *
            100).toFixed(2)}%`;
    }
    output() {
        var _a, _b;
        let tempStream;
        if (((_a = this.opts.stream) === null || _a === void 0 ? void 0 : _a.name) === 'file') {
            tempStream = new streams_1.FileStream(this.formatData, this.opts.stream.opts);
        }
        else if (((_b = this.opts.stream) === null || _b === void 0 ? void 0 : _b.name) === 'stdio') {
            tempStream = new streams_1.StdoutStream(this.formatData, this.opts.stream.opts);
        }
        else {
            throw new Error('参数错误');
        }
        tempStream.outToStream();
    }
}
exports.BaseProcess = BaseProcess;
__exportStar(require("./async"), exports);
__exportStar(require("./sync"), exports);
